// 4 Dec 2006 : Coding by Simone Mannori
//              ScicosHIL porting under Windows
//              First working code :)
//
// 7 Dec 2006 : Final 1.0 version 

//** -------------------------- DIGITAL OUTPUT -------------------------------
// 
// The two (Port A, Port B) digital I/O are bit addressable but each port MUST BE
// configured as ALL outputs OR inputs: no mixed input/output bit on the same port
// are allowed (see “usb-1208FS.c” and “usb-1208FS.h” for the details). Please group
// all the inputs on a port and all the outputs on the other port. Two output port 
// and two input ports configurations are allowed.
//
//**  Digital output block 

/* Include files */

#include <windows.h>                    /* Compiler's include files's */
#include <string.h>

#include <stdlib.h>
#include <stdio.h>

#include "scicos_block.h"  /* Specific for scicos block developement */
#include "machine.h"

#include "cbw.h"           /* Specific for MC USB devices  */ 
 
#define Init           4   /* Initialization */ 
#define OutputUpdate   1
#define StateUpdate    2
#define Integration    0
#define Mode           9    /* Zero Crossing */
#define EventScheduler 3
#define Ending         5   


 //** static structure inside block->work  
  typedef struct {
    int BoardNum    ;
    int UDStat      ;
    int PortNum     ;
	int Direction   ;
	int Chan        ;
	float Threshold ;
  } McUsbDigOut; 

// These two variables are used for the single bit handling 
// Storing the output state is possible to avoid the necessity to read the output register
// Read the output register is possible but is time consuming.
// Both output state variables are static and global because each blocks can modify the state
// of a single bit without changing the other output bits. 
static unsigned int PortAOutputState ; // State of the digital output port
static unsigned int PortBOutputState ;

void digital_output(scicos_block *block, int flag) 
{
  McUsbDigOut *DOdata ; //** define the local datastructure 
  
  unsigned int chan , portsel , portdir ; //**   
  unsigned short DataValue ;
  unsigned int bitmask   ;
  unsigned int portinput ; 

  double u[1] ; //** Scicos floating point input variable 
     
  switch(flag) {  
      
      case Init: //** Card and port init
        *block->work = (McUsbDigOut *) scicos_malloc(sizeof(McUsbDigOut));
        if (*block->work  == NULL ) 
          { //** in case of error exit 
	        set_block_error(-16); return; //** --> Exit point  
          }
	    DOdata = (McUsbDigOut *) *block->work; //** map the local structure in the scicos_block user data  
       
		DOdata->BoardNum = 0 ;
        DOdata->Threshold = (float) block->rpar[0];
	    chan    = block->ipar[0] ; // channel selection  
        portsel = block->ipar[1] ; //** 0->Port A, 1->Port B 
	    portdir = block->ipar[2] ; //** 0->ALL Inputs, 1->ALL Outputs  

		//** int cbDConfigPort(int BoardNum, int PortNum, int Direction)
		// int PortNum   : FIRSTPORTA FIRSTPORTB 
		// int Direction : DIGITALIN  DIGITALOUT
		
        DOdata->Chan = chan ; 

		if (portsel==0)
			 DOdata->PortNum = FIRSTPORTA ;
		if (portsel==1)
			 DOdata->PortNum = FIRSTPORTB ;
        
		if (portdir==0)
			 DOdata->Direction = DIGITALIN ;
		if (portdir==1)
			 DOdata->Direction = DIGITALOUT ;

        DOdata->UDStat = cbDConfigPort(DOdata->BoardNum, DOdata->PortNum, DOdata->Direction); 
        //** int cbDOut (int BoardNum, int PortNum, unsigned short DataValue)
        DataValue = 0 ; PortAOutputState = 0 ; PortBOutputState = 0 ;
        DOdata->UDStat = cbDOut (DOdata->BoardNum, DOdata->PortNum, DataValue); // clear all the outputs
		
	    // Commented out because is too time comsuming for the Init of many blocks
		// DOdata->UDStat = cbFlashLED (DOdata->BoardNum); //** Blink the greeen led ;)
      break; 
//==================================================================================      
      case StateUpdate:
         DOdata = (McUsbDigOut *) *block->work;
         u[0] = block->inptr[0][0] ; //** get the input from Scilab/Scicos       
	     
		 bitmask = 1 ;
	 
         if (DOdata->PortNum == FIRSTPORTA) 
		   {
		 	  if ( u[0] > DOdata->Threshold)
	               PortAOutputState = PortAOutputState | (bitmask << DOdata->Chan)  ; // :)
	           else
	               PortAOutputState = PortAOutputState & ~(bitmask << DOdata->Chan) ; // :( 
                    DOdata->UDStat = cbDOut(DOdata->BoardNum, DOdata->PortNum, PortAOutputState);
		   }

		 if (DOdata->PortNum == FIRSTPORTB) 
		   {
		 	  if ( u[0] > DOdata->Threshold)
	               PortBOutputState = PortBOutputState | (bitmask << DOdata->Chan)  ; // :)
	           else
	               PortBOutputState = PortBOutputState & ~(bitmask << DOdata->Chan) ; // :( 
			       DOdata->UDStat = cbDOut(DOdata->BoardNum, DOdata->PortNum, PortBOutputState);
		   }

      break;
//==================================================================================================      
      case Ending:
         DOdata = (McUsbDigOut *) *block->work;
     	     
		 PortAOutputState = 0 ; PortBOutputState = 0 ;
	 
         if (DOdata->PortNum == FIRSTPORTA) 
              DOdata->UDStat = cbDOut(DOdata->BoardNum, DOdata->PortNum, PortAOutputState);
		 
		 if (DOdata->PortNum == FIRSTPORTB) 
              DOdata->UDStat = cbDOut(DOdata->BoardNum, DOdata->PortNum, PortBOutputState); 


      break;	  
        
  } // close the switch
   
} // close the function 

